var fs = require('fs')
var path = require('path')
var _ = require('lodash')
var chalk = require('chalk')
var chokidar = require('chokidar')
var enableDestroy = require('server-destroy')
var is = require('./utils/is')
var load = require('./utils/load')
var jsonServer = require('../server')
var https = require('https');
var key = fs.readFileSync('./key/hacksparrow-key.pem');
var cert = fs.readFileSync('./key/hacksparrow-cert.pem')
var https_options = {
    key: key,
    cert: cert
};
function prettyPrint(argv, object, rules) {
    var host = argv.host === '0.0.0.0' ? 'localhost' : argv.host
    var port = argv.port
    var root = 'http://' + host + ':' + port
    if(argv.https){
        root = 'https://' + host + ':' + port
    }


    console.log()
    console.log(chalk.bold('  Resources'))
    for (var prop in object) {
        console.log('  ' + root + '/' + prop)
    }

    if (rules) {
        console.log()
        console.log(chalk.bold('  Other routes'))
        for (var rule in rules) {
            console.log('  ' + rule + ' -> ' + rules[rule])
        }
    }

    console.log()
    console.log(chalk.bold('  Home'))
    console.log('  ' + root)
    console.log()
}

function createApp(source, object, routes, argv) {
    var app = jsonServer.create()

    var router = jsonServer.router(
        is.JSON(source)
            ? source
            : object
    )

    var defaultsOpts;


    var defaults = jsonServer.defaults(defaultsOpts)
    app.use(defaults)

    router.db._.id = argv.id
    app.db = router.db
    app.use(router)

    return app
}

module.exports = function (argv) {
    var source = argv._[0]
    var app
    var server


    console.log()
    console.log(chalk.cyan('hello...'))

    function start(cb) {
        console.log()
        console.log(chalk.gray('  Loading', source))

        // Load JSON, JS or HTTP database
        load(source, function (err, data) {
            if (err) throw err


            console.log(chalk.gray('  Done'))
            var routes;
            // Create app and server
            app = createApp(source, data, routes, argv)
            if (argv.https) {
                server = https.createServer(https_options, app).listen(argv.port, argv.host);
            } else {
                server = app.listen(argv.port, argv.host)
            }


            // Enhance with a destroy function
            enableDestroy(server)

            // Display server informations
            prettyPrint(argv, data, routes)

            cb && cb()
        })
    }

    // Start server
    start(function () {

        console.log(chalk.blue('  Watching argv: ' + argv.watch));
        // Watch files
        if (argv.watch) {
            console.log(chalk.gray('  Watching...'))
            console.log()
            var source = argv._[0]


            // Watch .js or .json file
            // Since lowdb uses atomic writing, directory is watched instead of file
            chokidar
                .watch(path.dirname(source))
                .on('change', function (file) {
                    if (file === source) {
                        if (is.JSON(file)) {
                            var obj = JSON.parse(fs.readFileSync(file))
                            // Compare .json file content with in memory database
                            var isDatabaseDifferent = !_.isEqual(obj, app.db.getState())
                            if (isDatabaseDifferent) {
                                console.log(chalk.gray('  ' + file + ' has changed, reloading...'))
                                server && server.destroy()
                                start()
                            }
                        } else {
                            console.log(chalk.gray('  ' + file + ' has changed, reloading...'))
                            server && server.destroy()
                            start()
                        }
                    }
                })

        }
    })
}
