var express = require('express')
var bodyParser = require('body-parser')
var _ = require('lodash')
var _db = require('underscore-db')
var low = require('lowdb')
var fileAsync = require('lowdb/lib/file-async')
var singular = require('./singular')

module.exports = function (source) {
    // Create router
    var router = express.Router()

    // Add middlewares
    router.use(bodyParser.json({limit: '10mb', extended: false}))
    router.use(bodyParser.urlencoded({extended: false}))

    // Create database
    var db
    if (_.isObject(source)) {
        db = low()
        db.setState(source)
    } else {
        db = low(source, {storage: fileAsync})
    }

    // Add underscore-db methods to db
    db._.mixin(_db)


    // Expose database
    router.db = db

    // Expose render
    router.render = function (req, res) {
        res.jsonp(res.locals.data)
        console.log("data:"+JSON.stringify(res.locals.data));
    }

    // GET /db
    function showDatabase(req, res, next) {
        res.locals.data = db.getState()
        next()
    }

    router.get('/db', showDatabase)
    router.post('/db', showDatabase)

    // Create routes
    db.forEach(function (value, key) {
        console.log('value:' + JSON.stringify(value));
        console.log('key:' + key);

        router.use('/' + key, singular(db, key))


    }).value()

    router.use(function (req, res) {
        if (!res.locals.data) {
            res.status(404)
            res.locals.data = {}
        }

        router.render(req, res)
    })

    router.use(function (err, req, res, next) {
        console.error(err.stack)
        res.status(500).send(err.stack)
    })

    return router
}
